/**
    @file main.c

    Application main.

    Copyright (c) 2004-2005 Nokia Corporation.
	
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA
*/


#include <libintl.h>
#include <locale.h>
#include <gtk/gtk.h>
#include <libgnomevfs/gnome-vfs.h>
#include <libosso.h>
#include <signal.h>
#include <stdlib.h>

#include "rpc.h"
#include "ui/interface.h"
#include "utility.h"
#include "pdfviewer.h"
#include "configuration.h"
#include "settings.h"
#include "state_save.h"
#include "debug.h"

/*******************************************************************************
 **** Private data
 **/
static AppData *app_data = NULL;


/*******************************************************************************
 **** Private functions
 **/


/**

*/
static void free_recent_list_item(gpointer data, gpointer user_data)
{
    RecentItem *recent_item = (RecentItem*) data;
    
    g_return_if_fail(recent_item != NULL);

    /* cancel the monitor on the file */
    gnome_vfs_monitor_cancel(recent_item->monitor_handle);
    
    /* free the string used for uri */
    g_free(recent_item->uri);

    /* free the pointer itself */
    g_free(recent_item);
}


/**
   Free application data and substructures.
	
   @param app_data Application data
*/
static void free_app_data()
{
    if (!app_data)
	return;

    if (app_data->recent_list) {
        /* free each element */ 
        g_slist_foreach(app_data->recent_list, (GFunc) free_recent_list_item, NULL);
	/* free the list */
	g_slist_free(app_data->recent_list);
    }
    if (app_data->mmc_uri) {
        g_free(app_data->mmc_uri);
        app_data->mmc_uri = NULL;
    }
    if (app_data->app_ui_data) {
	g_free(app_data->app_ui_data);
	app_data->app_ui_data = NULL;
    }
    
    g_free(app_data);

}

/* Handler for POSIX signals */
static void signal_handler(int sig)
{
    static gboolean handling = FALSE;

    if (handling) {
        return;
    }
    handling = TRUE;
    
    /*
      if killable -> data already saved
    if(app_data)
      save_app_state(app_data);
    */
    
    /* gtk_main_quit is not safe here! There can be recursive loops
       that would all need to be quit. So we destroy everything,
       and exit. */

    if(app_data->app_ui_data->app)
      gtk_object_destroy(GTK_OBJECT(app_data->app_ui_data->app));

    if(app_data->gc_client)
      g_object_unref(app_data->gc_client);
    
    free_app_data();

    exit(EXIT_SUCCESS);
}


/*******************************************************************************
 **** Public function
 **/

/**
	Application main.
	Initializes internationalization, libosso, app and appview.
	Calls user interface creation functions and gtk_main.

	Follows the component initialization order of
	osso-filemanager main.
		
	@param argc Number of command line arguments
	@param argv Command line arguments
	@return 0 if successful; otherwise error code
*/
int main(int argc, char *argv[])
{
    gboolean          result;
    struct sigaction  sa;
    sigset_t          empty_mask;

    /* Allocate application data structures */
    app_data = g_new0(AppData, 1);
    if(app_data == NULL) {
      OSSO_LOG_CRIT("Failed memory allocation: AppData");
      exit(1);
    }
    
    /* there was no low memory ind from HW yet. */ 
    app_data->low_memory = FALSE;
    
    app_data->app_ui_data = g_new0(AppUIData, 1);
    if(app_data->app_ui_data == NULL) {
      OSSO_LOG_CRIT("Failed memory allocation: AppUIData");
      free_app_data();
      exit(1);
    }
    
    /* Add reference back to parent structure (AppData) */
    app_data->app_ui_data->app_data = app_data;
    
    /* Initialize osso */
    result = init_osso(app_data);
    if (!result) {
	free_app_data(app_data);
	return OSSO_ERROR;
    }

    /* Initialize GnomeVFS */
    result = gnome_vfs_init();

    if (!result) {
	OSSO_LOG_CRIT("Failed initializing GnomeVFS");
	deinit_osso(app_data);
	free_app_data(app_data);
	return OSSO_ERROR;
    }

    /* Initialize GConf and read application configuration */
    if (!init_settings(app_data))
    {
	OSSO_LOG_CRIT("Failed initializing GConf");
	deinit_osso(app_data);
	free_app_data(app_data);
	return OSSO_ERROR;
    }
    if (!get_configuration(argc, argv, app_data)) {
	OSSO_LOG_CRIT("Failed reading configuration");
	deinit_osso(app_data);
	free_app_data(app_data);
	return OSSO_ERROR;
    }
#ifdef ENABLE_NLS
    /* Initialize localization */
    /* Gettext does not seem to work properly without
       the following function call */
    setlocale(LC_ALL, "");

    bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
    bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
    textdomain(GETTEXT_PACKAGE);
#endif				/* ENABLE_NLS */

    /* Signals */
    sigemptyset (&empty_mask);
    sa.sa_handler = signal_handler;
    sa.sa_mask = empty_mask;
    sa.sa_flags = 0;
    sigaction (SIGTERM, &sa, NULL);
    sigaction (SIGINT, &sa, NULL);

    /* Initialize GTK */
    gtk_init(&argc, &argv);

    TDB(("10\n"));
    /* Create application UI */
    ui_create_main_window(app_data);
    
    /* Initialize engine */
    pdf_viewer_init(app_data->app_ui_data);

    TDB(("13\n"));
    gtk_main();

    TDB(("14\n"));
    /* Save configuration */
    save_configuration(app_data);

    /* Deinitialize engine */
    pdf_viewer_deinit();

    /* Deinitialize GConf */
    deinit_settings();

    /* Deinitialize osso */
    deinit_osso(app_data);

    /* Free memory */
    free_app_data(app_data);
    
    /* utility.c */
    free_mime_filters();
    free_application_mime_types();

    if (gnome_vfs_initialized()) {
        gnome_vfs_shutdown();
    }

    /* Exit successfully, regardless of any errors */
    return EXIT_SUCCESS;
}


/* EOF */
